varying highp vec4 Position0;
varying highp vec3 Normal0;
varying highp vec2 Texcoord0;

uniform bool        HasTexture;
uniform sampler2D   Texture0;

uniform highp vec4  EyePos;
uniform highp vec4  LightSourcePos;

uniform highp vec4  K_a;
uniform highp vec4  K_s;
uniform highp vec4  K_d;
uniform highp float Shininess;

uniform bool        TwoSided;
uniform highp vec4  Overlay;

uniform sampler2D    ShadowMap;
uniform highp mat4   LightSourceWVP;
uniform bool         CastShadowOnto;
uniform highp float  ShadowThreshold;

highp float SpecularComponent(highp vec4 lightSourcePos, highp vec4 eyePos, highp vec3 normal, highp vec4 pos, highp float shininess)
{
    // normal must be normalized
    highp vec3 _normal = normal;//normalize(normal);
    
    highp vec4 incident = pos-lightSourcePos;
    highp vec4 reflection = normalize(reflect(incident, vec4(_normal, 0)));
    
    highp vec4 lookat = normalize(eyePos-pos);
    
    highp float _s = max(0.0, dot(lookat, reflection));
    _s = pow(_s, shininess);
    
    return _s;
}

highp float IncidentDotNormal(highp vec4 lightSourcePos, highp vec3 normal, highp vec4 pos)
{
    highp float _d = dot(normalize(lightSourcePos-pos), vec4(normal,0));
    
    return _d;
}

highp vec4 ShadowComponent(highp vec4 position, highp float incidentDotNormal)
{
    // Shadow map computaion
    highp float shadowThreshold = ShadowThreshold * sin(acos(incidentDotNormal));
    
    highp vec4 posOnShadowMap = LightSourceWVP*position;//Position0;
    
    posOnShadowMap = posOnShadowMap / posOnShadowMap.a;
    
    // iOS depth render buffer did something so that the Y axis if flipped
    // if you choose to use that render buffer, use 1.0-posOnShadowMap
    posOnShadowMap.r = (posOnShadowMap.r+1.0)*0.5;
    //posOnShadowMap.g = (1.0-posOnShadowMap.g)*0.5;
    posOnShadowMap.g = (posOnShadowMap.g+1.0)*0.5;
    
    posOnShadowMap.b = (posOnShadowMap.b+1.0)*0.5;
    
    if(posOnShadowMap.r<0.0||posOnShadowMap.r>1.0||
       posOnShadowMap.g<0.0||posOnShadowMap.g>1.0||
       //z out of range
       posOnShadowMap.b<0.0||posOnShadowMap.b>1.0)
    {
        return vec4(1.0);
    }
    
    highp float depth = texture2D(ShadowMap, vec2(posOnShadowMap.r, posOnShadowMap.g)).r;
    
    if(CastShadowOnto)
    {
        if(posOnShadowMap.b - depth > shadowThreshold)
        {
            return vec4(0.0, 0.0, 0.0, 1.0);
        }
    }
    
    return vec4(1.0);
}

void main()
{
    highp vec4 lightSourcePos = LightSourcePos;
    highp vec4 eyePos = EyePos;
    
    highp vec4 ambient = vec4(0.0);
    
    if(HasTexture)
    {
        ambient = texture2D(Texture0, Texcoord0);
    }
    else
    {
        ambient = vec4(0.5,0.5,0.5,1);
    }
    
    highp vec4 ambient_comp = ambient;
    
    highp float incidentDotNormal = IncidentDotNormal(lightSourcePos, Normal0, Position0);
    
    if(TwoSided)
    {
        incidentDotNormal = abs(incidentDotNormal);
    }
    else
    {
        incidentDotNormal = max(0.0,incidentDotNormal);
    }
    
    highp vec4 diffuse_comp = ambient * incidentDotNormal; //vec4(Normal0,1);
    
    highp vec4 specular_comp = ambient * SpecularComponent(lightSourcePos, eyePos, Normal0, Position0, Shininess);
    
    gl_FragColor = K_a*ambient_comp + K_d*diffuse_comp+K_s*specular_comp;
    
    // combine with overlay
    gl_FragColor = gl_FragColor*(1.0-Overlay.a) + Overlay*Overlay.a;
    
    // consider shadow
    gl_FragColor = gl_FragColor*ShadowComponent(Position0, incidentDotNormal);
    
    gl_FragColor.a = 1.0;
}
