varying highp vec4 Position0;
varying highp vec2 Texcoord0;

uniform sampler2D   BumpMap;
uniform samplerCube ReflectionMap;
uniform sampler2D   RefractionMap;
uniform highp vec3  Color;
uniform highp float Transparency;

uniform highp vec2  FlowOffset;
uniform highp vec2  BumpMapSize;
uniform highp float Scale;
uniform highp float Height;

uniform highp mat4   matInvWVP;
uniform highp vec3   CameraRight;
uniform highp vec3   CameraUp;
uniform highp vec3   CameraLookAt;

uniform highp vec3   NormalDirection;

uniform sampler2D    ShadowMap;
uniform highp mat4   LightSourceWVP;
uniform bool         CastShadowOnto;
uniform bool         UseRefractionMap;
uniform highp float  ShadowThreshold;

uniform highp vec2   ViewportSize;

highp vec4 shadowComponent(highp vec4 position)
{
    // Shadow map computaion
    highp float shadowThreshold = ShadowThreshold;
    
    highp vec4 posOnShadowMap = LightSourceWVP*position;
    
    posOnShadowMap = posOnShadowMap / posOnShadowMap.a;
    
    if(posOnShadowMap.r<-1.0||posOnShadowMap.r>1.0||
       posOnShadowMap.g<-1.0||posOnShadowMap.g>1.0||
       posOnShadowMap.b<-1.0||posOnShadowMap.b>1.0)
    {
        // TODO: somehow GLSL compilation is having issue on A8 chips.
        // If using vec4(1.0), the function will return here.
        return vec4(1.0, 1.0, 0.999999, 1.0);
    }
    
    // iOS depth render buffer did something so that the Y axis if flipped
    // if you choose to use that render buffer, use 1.0-posOnShadowMap
    posOnShadowMap.r = (posOnShadowMap.r+1.0)*0.5;
    //posOnShadowMap.g = (1.0-posOnShadowMap.g)*0.5;
    posOnShadowMap.g = (posOnShadowMap.g+1.0)*0.5;
    
    posOnShadowMap.b = (posOnShadowMap.b+1.0)*0.5;
    
    highp float depth = texture2D(ShadowMap, vec2(posOnShadowMap.r, posOnShadowMap.g)).r;
    
    if(CastShadowOnto)
    {
        if(posOnShadowMap.b - depth > shadowThreshold)
        {
            return vec4(0.7, 0.7, 0.7, 1.0);
        }
    }
    
    return vec4(1.0);
}

highp vec3 UncompressNormalDir(highp vec3 dir)
{
    return normalize(dir*2.0-vec3(1.0));
}

highp vec4 GetNormalDir()
{
    if(UseRefractionMap)
    {
        highp vec4 bumpDir = texture2D(BumpMap, vec2(gl_FragCoord.x / ViewportSize.x, gl_FragCoord.y / ViewportSize.y));
        
        return vec4(normalize(UncompressNormalDir(vec3(bumpDir))), bumpDir.w);
    }
    else
    {
        highp vec2 offset = FlowOffset;
        highp vec2 scaled_texcoord = Texcoord0*Scale;
        
        highp vec2 bumpTexcoord0 = vec2(fract(scaled_texcoord.x+offset.x), fract(scaled_texcoord.y+offset.y));
        
        highp vec4 bumpDir = texture2D(BumpMap, bumpTexcoord0);
        
        return vec4(normalize(vec3(bumpDir))*Height, 0.0);
    }
}

void main()
{
    const highp float EPSILON = 0.001;
    
    highp vec3 lookatDir = vec3(matInvWVP*vec4(gl_FragCoord.x, gl_FragCoord.y, 0, 1.0));
    //lookatDir = normalize(lookatDir);
    
    highp vec4 bumpDir = GetNormalDir();
    
    highp vec3 normal = vec3(bumpDir);
    
    highp vec3 reflectDir = reflect(lookatDir, normal);
    
    highp vec4 reflectionColor = textureCube(ReflectionMap, reflectDir);
    
    //------------------------------------------------------------------------------------------
    // check if it is within the reflection mask
    highp float refractionPortion = 0.0;
    
    highp vec4 refractionColor = vec4(0);
    
    if(UseRefractionMap)
    {
        highp vec3 _lookatDir = CameraLookAt;
        highp vec3 refractDir = refract(_lookatDir, normal, 1.0/1.3);
        
        highp vec2 texcoordOnReflection = vec2(gl_FragCoord.x/ViewportSize.x, gl_FragCoord.y/ViewportSize.y);
        highp vec4 valueOnRefraction = texture2D(RefractionMap, texcoordOnReflection);
        highp float depthOnRefraction = valueOnRefraction.w;
        
        // decrease sigma to achieve more dramatic wave
        highp float sigma = 0.01;
        highp float zdistance = bumpDir.w-depthOnRefraction;
        
        highp float zOffset = Height*(1.0-exp(-zdistance*zdistance/sigma/sigma));
        
        highp vec3 lookatOffset = refractDir - _lookatDir;
        // TODO: normal direction is fixed now
        lookatOffset = lookatOffset - dot(lookatOffset, NormalDirection) * NormalDirection;
        
        highp vec2 dirOffset = vec2(dot(lookatOffset, CameraRight), dot(lookatOffset, CameraUp));
        
        highp vec2 _texcoord = vec2(gl_FragCoord.x/ViewportSize.x+dirOffset.x*zOffset, gl_FragCoord.y/ViewportSize.y+dirOffset.y*zOffset);
        
        highp vec4 maskValue = texture2D(BumpMap, _texcoord);
        
        highp vec4 enviromentColor = textureCube(ReflectionMap, refractDir);
        
        highp float enviromentPortion = 0.3;
        // if refraction map has valid value.
        if(maskValue.r > EPSILON || maskValue.g > EPSILON || maskValue.b > EPSILON)
        {
            refractionColor     = texture2D(RefractionMap, _texcoord);
            refractionColor.a   = 1.0;
            
            refractionColor     = refractionColor*(1.0-enviromentPortion) + enviromentColor*enviromentPortion;
        }
        else
        {
            valueOnRefraction.a = 1.0;
            refractionColor     = valueOnRefraction*(1.0-enviromentPortion) + enviromentColor*enviromentPortion;
        }
        
        refractionPortion = 0.8;
    }
    //------------------------------------------------------------------------------------------
    
    gl_FragColor = vec4(Color, 1.0)*(1.0-Transparency)+Transparency*(reflectionColor*(1.0-refractionPortion)+refractionColor*refractionPortion);
    
    //gl_FragColor = reflectionColor;
    
    // TODO: hard coded scale
    // TODO: water gradient need to be passed in
    highp vec4 waterGradient = vec4(1, 0, 1, 0);
    gl_FragColor = gl_FragColor*shadowComponent(Position0+Height*vec4(normal, 0)*waterGradient);
}
